@@ -16,29 +16,74 @@ namespace GPBoost {
 	REModel::REModel() {
 	}
 
-	REModel::REModel(data_size_t num_data, const gp_id_t* cluster_ids_data, const char* re_group_data, data_size_t num_re_group,
-		const double* re_group_rand_coef_data, const int32_t* ind_effect_group_rand_coef, data_size_t num_re_group_rand_coef,
-		data_size_t num_gp, const double* gp_coords_data, int dim_gp_coords, const double* gp_rand_coef_data, data_size_t num_gp_rand_coef,
-		const char* cov_fct, double cov_fct_shape, bool vecchia_approx, int num_neighbors, const char* vecchia_ordering,
-		const char* vecchia_pred_type, int num_neighbors_pred, const char* likelihood) {
+	REModel::REModel(data_size_t num_data,
+		const gp_id_t* cluster_ids_data,
+		const char* re_group_data,
+		data_size_t num_re_group,
+		const double* re_group_rand_coef_data,
+		const int32_t* ind_effect_group_rand_coef,
+		data_size_t num_re_group_rand_coef,
+		data_size_t num_gp,
+		const double* gp_coords_data,
+		int dim_gp_coords,
+		const double* gp_rand_coef_data,
+		data_size_t num_gp_rand_coef,
+		const char* cov_fct,
+		double cov_fct_shape,
+		bool vecchia_approx,
+		int num_neighbors,
+		const char* vecchia_ordering,
+		const char* vecchia_pred_type,
+		int num_neighbors_pred,
+		const char* likelihood) {
 		if ((num_gp + num_gp_rand_coef) == 0) {
 			sparse_ = true;
 			re_model_sp_ = std::unique_ptr<REModelTemplate<sp_mat_t, chol_sp_mat_t>>(new REModelTemplate<sp_mat_t, chol_sp_mat_t>(
-				num_data, cluster_ids_data, re_group_data, num_re_group,
-				re_group_rand_coef_data, ind_effect_group_rand_coef, num_re_group_rand_coef,
-				num_gp, gp_coords_data, dim_gp_coords, gp_rand_coef_data, num_gp_rand_coef,
-				cov_fct, cov_fct_shape, vecchia_approx, num_neighbors, vecchia_ordering,
-				vecchia_pred_type, num_neighbors_pred, likelihood));
+				num_data,
+				cluster_ids_data,
+				re_group_data,
+				num_re_group,
+				re_group_rand_coef_data,
+				ind_effect_group_rand_coef,
+				num_re_group_rand_coef,
+				num_gp,
+				gp_coords_data,
+				dim_gp_coords,
+				gp_rand_coef_data,
+				num_gp_rand_coef,
+				cov_fct,
+				cov_fct_shape,
+				vecchia_approx,
+				num_neighbors, 
+				vecchia_ordering,
+				vecchia_pred_type,
+				num_neighbors_pred,
+				likelihood));
 			num_cov_pars_ = re_model_sp_->num_cov_par_;
 		}
 		else {
 			sparse_ = false;
-			re_model_den_ = std::unique_ptr <REModelTemplate< den_mat_t, chol_den_mat_t >>(new REModelTemplate<den_mat_t, chol_den_mat_t>(
-				num_data, cluster_ids_data, re_group_data, num_re_group,
-				re_group_rand_coef_data, ind_effect_group_rand_coef, num_re_group_rand_coef,
-				num_gp, gp_coords_data, dim_gp_coords, gp_rand_coef_data, num_gp_rand_coef,
-				cov_fct, cov_fct_shape, vecchia_approx, num_neighbors, vecchia_ordering,
-				vecchia_pred_type, num_neighbors_pred, likelihood));
+			re_model_den_ = std::unique_ptr <REModelTemplate< den_mat_t, chol_den_mat_t>>(new REModelTemplate<den_mat_t, chol_den_mat_t>(
+				num_data,
+				cluster_ids_data,
+				re_group_data,
+				num_re_group,
+				re_group_rand_coef_data,
+				ind_effect_group_rand_coef,
+				num_re_group_rand_coef,
+				num_gp,
+				gp_coords_data,
+				dim_gp_coords,
+				gp_rand_coef_data,
+				num_gp_rand_coef,
+				cov_fct,
+				cov_fct_shape,
+				vecchia_approx,
+				num_neighbors,
+				vecchia_ordering,
+				vecchia_pred_type,
+				num_neighbors_pred,
+				likelihood));
 			num_cov_pars_ = re_model_den_->num_cov_par_;
 		}
 		if (!GaussLikelihood()) {
@@ -189,30 +234,54 @@ namespace GPBoost {
 		}
 		if (sparse_) {
 			re_model_sp_->OptimLinRegrCoefCovPar(y_data,
-				nullptr, 0,
-				cov_pars_.data(), nullptr,
+				nullptr,
+				0,
+				cov_pars_.data(),
+				nullptr,
 				num_it_,
-				cov_pars_.data(), nullptr,
-				1, lr_cov_, 1, acc_rate_cov_, momentum_offset_,
-				max_iter_, delta_rel_conv_,
-				use_nesterov_acc_, nesterov_schedule_version_,
-				optimizer_cov_pars_, "none",
-				std_dev_cov_par, nullptr, calc_std_dev_,
+				cov_pars_.data(),
+				nullptr,
+				1,
+				lr_cov_,
+				1,
+				acc_rate_cov_,
+				momentum_offset_,
+				max_iter_,
+				delta_rel_conv_,
+				use_nesterov_acc_,
+				nesterov_schedule_version_,
+				optimizer_cov_pars_,
+				"none",
+				std_dev_cov_par,
+				nullptr,
+				calc_std_dev_,
 				convergence_criterion_,
 				fixed_effects,
 				true);
 		}
 		else {
 			re_model_den_->OptimLinRegrCoefCovPar(y_data,
-				nullptr, 0,
-				cov_pars_.data(), nullptr,
+				nullptr,
+				0,
+				cov_pars_.data(),
+				nullptr,
 				num_it_,
-				cov_pars_.data(), nullptr,
-				0., lr_cov_, 0., acc_rate_cov_, momentum_offset_,
-				max_iter_, delta_rel_conv_,
-				use_nesterov_acc_, nesterov_schedule_version_,
-				optimizer_cov_pars_, "none",
-				std_dev_cov_par, nullptr, calc_std_dev_,
+				cov_pars_.data(),
+				nullptr,
+				1,
+				lr_cov_,
+				1,
+				acc_rate_cov_,
+				momentum_offset_,
+				max_iter_,
+				delta_rel_conv_,
+				use_nesterov_acc_,
+				nesterov_schedule_version_,
+				optimizer_cov_pars_,
+				"none",
+				std_dev_cov_par,
+				nullptr,
+				calc_std_dev_,
 				convergence_criterion_,
 				fixed_effects,
 				true);
@@ -281,30 +350,54 @@ namespace GPBoost {
 		init_score[0] = 0.;
 		if (sparse_) {
 			re_model_sp_->OptimLinRegrCoefCovPar(nullptr,
-				covariate_data.data(), 1,
-				cov_pars_.data(), init_score,
+				covariate_data.data(),
+				1,
+				cov_pars_.data(),
+				init_score,
 				num_it_,
-				cov_pars_.data(), init_score,
-				lr_coef_, 0., acc_rate_coef_, 0., momentum_offset_,
-				max_iter_, delta_rel_conv_,
-				use_nesterov_acc_, nesterov_schedule_version_,
-				optimizer_cov_pars_, optimizer_coef_,
-				nullptr, nullptr, false,
+				cov_pars_.data(),
+				init_score,
+				lr_coef_,
+				lr_cov_,
+				acc_rate_coef_,
+				acc_rate_cov_,
+				momentum_offset_,
+				max_iter_,
+				delta_rel_conv_,
+				use_nesterov_acc_,
+				nesterov_schedule_version_,
+				optimizer_cov_pars_,
+				optimizer_coef_,
+				nullptr,
+				nullptr,
+				false,
 				convergence_criterion_,
 				nullptr,
 				false);
 		}
 		else {
 			re_model_den_->OptimLinRegrCoefCovPar(nullptr,
-				covariate_data.data(), 1,
-				cov_pars_.data(), init_score,
+				covariate_data.data(),
+				1,
+				cov_pars_.data(),
+				init_score,
 				num_it_,
-				cov_pars_.data(), init_score,
-				lr_coef_, 0., acc_rate_coef_, 0., momentum_offset_,
-				max_iter_, delta_rel_conv_,
-				use_nesterov_acc_, nesterov_schedule_version_,
-				optimizer_cov_pars_, optimizer_coef_,
-				nullptr, nullptr, false,
+				cov_pars_.data(),
+				init_score,
+				lr_coef_,
+				lr_cov_,
+				acc_rate_coef_,
+				acc_rate_cov_,
+				momentum_offset_,
+				max_iter_,
+				delta_rel_conv_,
+				use_nesterov_acc_,
+				nesterov_schedule_version_,
+				optimizer_cov_pars_,
+				optimizer_coef_,
+				nullptr,
+				nullptr,
+				false,
 				convergence_criterion_,
 				nullptr,
 				false);
@@ -349,7 +442,9 @@ namespace GPBoost {
 				re_model_den_->EvalLAApproxNegLogLikelihood(y_data, cov_pars_trafo.data(), negll, fixed_effects, InitializeModeCovMat, CalcModePostRandEff_already_done);
 			}
 		}
-		covariance_matrix_has_been_factorized_ = true;
+		covariance_matrix_has_been_factorized_ = false;
+		//set to false as otherwise the covariance is not factorized for prediction for Gaussian data and this can lead to problems 
+		//(e.g. fitting model with covariates, then calling likelihood without covariates, then making prediction with covariates)
 	}
 
 	void REModel::CalcGradient(double* y, const double* fixed_effects, bool calc_cov_factor) {
