@@ -105,7 +105,7 @@ test_that("gpb.load() gives the expected error messages given different incorrec
         , nrounds = 2L
         , objective = "binary"
     )
-
+    
     # you have to give model_str or filename
     expect_error({
         gpb.load()
@@ -113,7 +113,7 @@ test_that("gpb.load() gives the expected error messages given different incorrec
     expect_error({
         gpb.load(filename = NULL, model_str = NULL)
     }, regexp = "either filename or model_str must be given")
-
+    
     # if given, filename should be a string that points to an existing file
     model_file <- tempfile(fileext = ".model")
     expect_error({
@@ -126,12 +126,12 @@ test_that("gpb.load() gives the expected error messages given different incorrec
     expect_error({
         gpb.load(filename = file_to_check)
     }, regexp = "passed to filename does not exist")
-
+    
     # if given, model_str should be a string
     expect_error({
         gpb.load(model_str = c(4.0, 5.0, 6.0))
     }, regexp = "model_str should be character")
-
+    
 })
 
 test_that("Loading a Booster from a text file works", {
@@ -149,16 +149,16 @@ test_that("Loading a Booster from a text file works", {
         , objective = "binary"
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
-
+    
     pred <- predict(bst, test$data)
     model_file <- tempfile(fileext = ".model")
     gpb.save(bst, model_file)
-
+    
     # finalize the booster and destroy it so you know we aren't cheating
     bst$finalize()
     expect_null(bst$.__enclos_env__$private$handle)
     rm(bst)
-
+    
     bst2 <- gpb.load(
         filename = model_file
     )
@@ -173,7 +173,7 @@ test_that("boosters with linear models at leaves can be written to text file and
         data = X
         , label = labels
     )
-
+    
     params <- list(
         objective = "regression"
         , verbose = -1L
@@ -181,14 +181,14 @@ test_that("boosters with linear models at leaves can be written to text file and
         , seed = 0L
         , num_leaves = 2L
     )
-
+    
     bst <- gpb.train(
         data = dtrain
         , nrounds = 10L
         , params = params
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
-
+    
     # save predictions, then write the model to a file and destroy it in R
     preds <- predict(bst, X)
     model_file <- tempfile(fileext = ".model")
@@ -196,7 +196,7 @@ test_that("boosters with linear models at leaves can be written to text file and
     bst$finalize()
     expect_null(bst$.__enclos_env__$private$handle)
     rm(bst)
-
+    
     # load the booster and make predictions...should be the same
     bst2 <- gpb.load(
         filename = model_file
@@ -221,15 +221,15 @@ test_that("Loading a Booster from a string works", {
         , objective = "binary"
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
-
+    
     pred <- predict(bst, test$data)
     model_string <- bst$save_model_to_string()
-
+    
     # finalize the booster and destroy it so you know we aren't cheating
     bst$finalize()
     expect_null(bst$.__enclos_env__$private$handle)
     rm(bst)
-
+    
     bst2 <- gpb.load(
         model_str = model_string
     )
@@ -252,16 +252,16 @@ test_that("If a string and a file are both passed to gpb.load() the file is used
         , objective = "binary"
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
-
+    
     pred <- predict(bst, test$data)
     model_file <- tempfile(fileext = ".model")
     gpb.save(bst, model_file)
-
+    
     # finalize the booster and destroy it so you know we aren't cheating
     bst$finalize()
     expect_null(bst$.__enclos_env__$private$handle)
     rm(bst)
-
+    
     bst2 <- gpb.load(
         filename = model_file
         , model_str = 4.0
@@ -341,14 +341,14 @@ test_that("Booster$rollback_one_iter() should work as expected", {
     expect_true(gpboost:::gpb.is.Booster(bst))
     logloss <- bst$eval_train()[[1L]][["value"]]
     expect_equal(logloss, 0.01991487)
-
+    
     x <- bst$rollback_one_iter()
-
+    
     # rollback_one_iter() should return a booster and modify the original
     # booster in place
     expect_true(gpboost:::gpb.is.Booster(x))
     expect_equal(bst$current_iter(), nrounds - 1L)
-
+    
     # score should now come from the model as of 4 iterations
     logloss <- bst$eval_train()[[1L]][["value"]]
     expect_equal(logloss, 0.03150228)
@@ -358,7 +358,7 @@ test_that("Booster$update() passing a train_set works as expected", {
     set.seed(708L)
     data(agaricus.train, package = "gpboost")
     nrounds <- 2L
-
+    
     # train with 2 rounds and then update
     bst <- gpboost(
         data = as.matrix(agaricus.train$data)
@@ -378,7 +378,7 @@ test_that("Booster$update() passing a train_set works as expected", {
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
     expect_equal(bst$current_iter(), nrounds + 1L)
-
+    
     # train with 3 rounds directly
     bst2 <- gpboost(
         data = as.matrix(agaricus.train$data)
@@ -390,7 +390,7 @@ test_that("Booster$update() passing a train_set works as expected", {
     )
     expect_true(gpboost:::gpb.is.Booster(bst2))
     expect_equal(bst2$current_iter(), nrounds +  1L)
-
+    
     # model with 2 rounds + 1 update should be identical to 3 rounds
     expect_equal(bst2$eval_train()[[1L]][["value"]], 0.04806585)
     expect_equal(bst$eval_train()[[1L]][["value"]], bst2$eval_train()[[1L]][["value"]])
@@ -400,7 +400,7 @@ test_that("Booster$update() throws an informative error if you provide a non-Dat
     set.seed(708L)
     data(agaricus.train, package = "gpboost")
     nrounds <- 2L
-
+    
     # train with 2 rounds and then update
     bst <- gpboost(
         data = as.matrix(agaricus.train$data)
@@ -439,7 +439,7 @@ test_that("Booster should store parameters and Booster$reset_parameter() should
         , train_set = dtrain
     )
     expect_identical(bst$params, params)
-
+    
     params[["bagging_fraction"]] <- 0.9
     ret_bst <- bst$reset_parameter(params = params)
     expect_identical(ret_bst$params, params)
@@ -473,7 +473,7 @@ test_that("Booster$params should include dataset params, before and after Booste
             , max_bin = 17L
         )
     )
-
+    
     params[["bagging_fraction"]] <- 0.9
     ret_bst <- bst$reset_parameter(params = params)
     expected_params <- list(
@@ -488,57 +488,59 @@ test_that("Booster$params should include dataset params, before and after Booste
 
 context("save_model")
 
-test_that("Saving a model with different feature importance types works", {
-    set.seed(708L)
-    data(agaricus.train, package = "gpboost")
-    train <- agaricus.train
-    bst <- gpboost(
-        data = as.matrix(train$data)
-        , label = train$label
-        , num_leaves = 4L
-        , learning_rate = 1.0
-        , nrounds = 2L
-        , objective = "binary"
-    )
-    expect_true(gpboost:::gpb.is.Booster(bst))
-
-    .feat_importance_from_string <- function(model_string) {
-        file_lines <- strsplit(model_string, "\n")[[1L]]
-        start_indx <- which(grepl("^feature_importances\\:$", file_lines)) + 1L
-        blank_line_indices <- which(file_lines == "")
-        end_indx <- blank_line_indices[blank_line_indices > start_indx][1L] - 1L
-        importances <- file_lines[start_indx: end_indx]
-        return(importances)
-    }
-
-    GAIN_IMPORTANCE <- 1L
-    model_string <- bst$save_model_to_string(feature_importance_type = GAIN_IMPORTANCE)
-    expect_equal(
-        .feat_importance_from_string(model_string)
-        , c(
-            "odor=none=4010"
-            , "stalk-root=club=1163"
-            , "stalk-root=rooted=573"
-            , "stalk-surface-above-ring=silky=450"
-            , "spore-print-color=green=397"
-            , "gill-color=buff=281"
+if(Sys.getenv("GPBOOST_ALL_TESTS") == "GPBOOST_ALL_TESTS"){
+    test_that("Saving a model with different feature importance types works", {
+        set.seed(708L)
+        data(agaricus.train, package = "gpboost")
+        train <- agaricus.train
+        bst <- gpboost(
+            data = as.matrix(train$data)
+            , label = train$label
+            , num_leaves = 4L
+            , learning_rate = 1.0
+            , nrounds = 2L
+            , objective = "binary"
         )
-    )
-
-    SPLIT_IMPORTANCE <- 0L
-    model_string <- bst$save_model_to_string(feature_importance_type = SPLIT_IMPORTANCE)
-    expect_equal(
-        .feat_importance_from_string(model_string)
-        , c(
-            "odor=none=1"
-            , "gill-color=buff=1"
-            , "stalk-root=club=1"
-            , "stalk-root=rooted=1"
-            , "stalk-surface-above-ring=silky=1"
-            , "spore-print-color=green=1"
+        expect_true(gpboost:::gpb.is.Booster(bst))
+        
+        .feat_importance_from_string <- function(model_string) {
+            file_lines <- strsplit(model_string, "\n")[[1L]]
+            start_indx <- which(grepl("^feature_importances\\:$", file_lines)) + 1L
+            blank_line_indices <- which(file_lines == "")
+            end_indx <- blank_line_indices[blank_line_indices > start_indx][1L] - 1L
+            importances <- file_lines[start_indx: end_indx]
+            return(importances)
+        }
+        
+        GAIN_IMPORTANCE <- 1L
+        model_string <- bst$save_model_to_string(feature_importance_type = GAIN_IMPORTANCE)
+        expect_equal(
+            .feat_importance_from_string(model_string)
+            , c(
+                "odor=none=4010"
+                , "stalk-root=club=1163"
+                , "stalk-root=rooted=573"
+                , "stalk-surface-above-ring=silky=450"
+                , "spore-print-color=green=397"
+                , "gill-color=buff=281"
+            )
         )
-    )
-})
+        
+        SPLIT_IMPORTANCE <- 0L
+        model_string <- bst$save_model_to_string(feature_importance_type = SPLIT_IMPORTANCE)
+        expect_equal(
+            .feat_importance_from_string(model_string)
+            , c(
+                "odor=none=1"
+                , "gill-color=buff=1"
+                , "stalk-root=club=1"
+                , "stalk-root=rooted=1"
+                , "stalk-surface-above-ring=silky=1"
+                , "spore-print-color=green=1"
+            )
+        )
+    })
+}
 
 # test_that("Saving a model with unknown importance type fails", {
 #     testthat::skip("Skipping this test because it causes issues for valgrind")
@@ -586,23 +588,23 @@ test_that("all parameters are stored correctly with save_model_to_string()", {
         , nrounds = nrounds
         , verbose = 0L
     )
-
+    
     model_str <- bst$save_model_to_string()
     params_in_file <- .params_from_model_string(model_str = model_str)
-
+    
     # parameters should match what was passed from the R package
     expect_equal(sum(grepl(pattern = "^\\[metric\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == "[metric: l2]"), 1L)
-
+    
     expect_equal(sum(grepl(pattern = "^\\[num_iterations\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == "[num_iterations: 4]"), 1L)
-
+    
     expect_equal(sum(grepl(pattern = "^\\[objective\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == "[objective: regression]"), 1L)
-
+    
     expect_equal(sum(grepl(pattern = "^\\[verbosity\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == "[verbosity: 0]"), 1L)
-
+    
     # early stopping should be off by default
     expect_equal(sum(grepl(pattern = "^\\[early_stopping_round\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == "[early_stopping_round: 0]"), 1L)
@@ -617,18 +619,18 @@ test_that("early_stopping, num_iterations are stored correctly in model string e
         data = matrix(rnorm(500L), nrow = 100L)
         , label = rnorm(100L)
     )
-
+    
     # num_iterations values (all different)
     num_iterations <- 4L
     num_boost_round <- 2L
     n_iter <- 3L
     nrounds_kwarg <- 6L
-
+    
     # early_stopping_round values (all different)
     early_stopping_round <- 2L
     early_stopping_round_kwarg <- 3L
     n_iter_no_change <- 4L
-
+    
     params <- list(
         objective = "regression"
         , metric = "l2"
@@ -638,7 +640,7 @@ test_that("early_stopping, num_iterations are stored correctly in model string e
         , early_stopping_round = early_stopping_round
         , n_iter_no_change = n_iter_no_change
     )
-
+    
     bst <- gpb.train(
         params = params
         , data = dtrain
@@ -649,22 +651,22 @@ test_that("early_stopping, num_iterations are stored correctly in model string e
         )
         , verbose = 0L
     )
-
+    
     model_str <- bst$save_model_to_string()
     params_in_file <- .params_from_model_string(model_str = model_str)
-
+    
     # parameters should match what was passed from the R package, and the "main" (non-alias)
     # params values in `params` should be preferred to keyword argumentts or aliases
     expect_equal(sum(grepl(pattern = "^\\[num_iterations\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == sprintf("[num_iterations: %s]", num_iterations)), 1L)
     expect_equal(sum(grepl(pattern = "^\\[early_stopping_round\\:", x = params_in_file)), 1L)
     expect_equal(sum(params_in_file == sprintf("[early_stopping_round: %s]", early_stopping_round)), 1L)
-
+    
     # none of the aliases shouold have been written to the model file
     expect_equal(sum(grepl(pattern = "^\\[num_boost_round\\:", x = params_in_file)), 0L)
     expect_equal(sum(grepl(pattern = "^\\[n_iter\\:", x = params_in_file)), 0L)
     expect_equal(sum(grepl(pattern = "^\\[n_iter_no_change\\:", x = params_in_file)), 0L)
-
+    
 })
 
 # this is almost identical to the test above it, but for gpb.cv(). A lot of code
@@ -675,18 +677,18 @@ test_that("gpb.cv() correctly handles passing through params to the model file",
         data = matrix(rnorm(500L), nrow = 100L)
         , label = rnorm(100L)
     )
-
+    
     # num_iterations values (all different)
     num_iterations <- 4L
     num_boost_round <- 2L
     n_iter <- 3L
     nrounds_kwarg <- 6L
-
+    
     # early_stopping_round values (all different)
     early_stopping_round <- 2L
     early_stopping_round_kwarg <- 3L
     n_iter_no_change <- 4L
-
+    
     params <- list(
         objective = "regression"
         , metric = "l2"
@@ -696,7 +698,7 @@ test_that("gpb.cv() correctly handles passing through params to the model file",
         , early_stopping_round = early_stopping_round
         , n_iter_no_change = n_iter_no_change
     )
-
+    
     cv_bst <- gpb.cv(
         params = params
         , data = dtrain
@@ -705,24 +707,24 @@ test_that("gpb.cv() correctly handles passing through params to the model file",
         , nfold = 3L
         , verbose = 0L
     )
-
+    
     for (bst in cv_bst$boosters) {
         model_str <- bst[["booster"]]$save_model_to_string()
         params_in_file <- .params_from_model_string(model_str = model_str)
-
+        
         # parameters should match what was passed from the R package, and the "main" (non-alias)
         # params values in `params` should be preferred to keyword argumentts or aliases
         expect_equal(sum(grepl(pattern = "^\\[num_iterations\\:", x = params_in_file)), 1L)
         expect_equal(sum(params_in_file == sprintf("[num_iterations: %s]", num_iterations)), 1L)
         expect_equal(sum(grepl(pattern = "^\\[early_stopping_round\\:", x = params_in_file)), 1L)
         expect_equal(sum(params_in_file == sprintf("[early_stopping_round: %s]", early_stopping_round)), 1L)
-
+        
         # none of the aliases shouold have been written to the model file
         expect_equal(sum(grepl(pattern = "^\\[num_boost_round\\:", x = params_in_file)), 0L)
         expect_equal(sum(grepl(pattern = "^\\[n_iter\\:", x = params_in_file)), 0L)
         expect_equal(sum(grepl(pattern = "^\\[n_iter_no_change\\:", x = params_in_file)), 0L)
     }
-
+    
 })
 
 context("saveRDS.gpb.Booster() and readRDS.gpb.Booster()")
@@ -747,7 +749,7 @@ test_that("params (including dataset params) should be stored in .rds file for B
     )
     bst_file <- tempfile(fileext = ".rds")
     saveRDS.gpb.Booster(bst, file = bst_file)
-
+    
     bst_from_file <- readRDS.gpb.Booster(file = bst_file)
     expect_identical(
         bst_from_file$params
@@ -767,7 +769,7 @@ test_that("boosters with linear models at leaves can be written to RDS and re-lo
         data = X
         , label = labels
     )
-
+    
     params <- list(
         objective = "regression"
         , verbose = -1L
@@ -775,14 +777,14 @@ test_that("boosters with linear models at leaves can be written to RDS and re-lo
         , seed = 0L
         , num_leaves = 2L
     )
-
+    
     bst <- gpb.train(
         data = dtrain
         , nrounds = 10L
         , params = params
     )
     expect_true(gpboost:::gpb.is.Booster(bst))
-
+    
     # save predictions, then write the model to a file and destroy it in R
     preds <- predict(bst, X)
     model_file <- tempfile(fileext = ".rds")
@@ -790,7 +792,7 @@ test_that("boosters with linear models at leaves can be written to RDS and re-lo
     bst$finalize()
     expect_null(bst$.__enclos_env__$private$handle)
     rm(bst)
-
+    
     # load the booster and make predictions...should be the same
     bst2 <- readRDS.gpb.Booster(file = model_file)
     preds2 <- predict(bst2, X)
